/* Sketch a corredo delle risorse online del libro di testo
   Onelio Bertazioli
   Telecomunicazioni - Seconda edizione
   Zanichelli*/
   
// MISURA DI TEMPERATURA ED UMIDITA' CON IL SENSORE DHT 11
/* includere le librerie 
-> DHT Sensor Library: https://github.com/adafruit/DHT-sensor-library
-> Adafruit Unified Sensor Lib: https://github.com/adafruit/Adafruit_Sensor
 */
//collegare il pin dati del sensore al pin digitale 2; 
//NB: collegare +VCC a 3.3 V per seeeduino!!

#include <DHT.h>
#include <DHT_U.h>

#define DHTTYPE    DHT11
#define DHTPIN 2   // Collega il pin segnale al pin 2 
//alimentare a 3,3 V!!!!!!

DHT dht(DHTPIN, DHTTYPE);

#include <LoRaWan.h>  
//libreria LoRaWan che gestisce la comunicazione via LoRaWAN

float temp; //temperatura
float umid; //umidità

char buffer[256];


//----------------------------------------
void setup()
{
    dht.begin();  // Inizializza il sensore

    SerialUSB.begin(115200);
    while(!SerialUSB);
    
    lora.init(); //inizializza il modem LoRa

    //manda sul monitor seriali le informazionisulla configurazione di seeduino LoRaWAN 
    memset(buffer, 0, 256);
    lora.getVersion(buffer, 256, 1);
    SerialUSB.print(buffer);   
    memset(buffer, 0, 256);
    lora.getId(buffer, 256, 1);
    SerialUSB.print(buffer);
    
    lora.setKey("lavostraNWKKEY", "lavostraAPPSKEY", NULL);
    //Inserire le proprie Chiavi NWKKEY, APPSKEY, (NULL significa che APPKEY NON viene usata nella modalità APB, Activation By Personalization) da configurare 
    //sia su seeduino sia su loriot.io per configurare una sessione ABP (Activate By Personalization) su loriot.io)
    
    lora.setDeciveMode(LWABP);
    //setta la modalità ABP (Activation By Personalization) che fa unire direttamene un end-devicea una rete lorawan saltando la fase di JOIN della modalità OTAA
    
    lora.setDataRate(DR0, EU868); //setta il data rate DR0 (250 bit/s)
    
    lora.setChannel(0, 868.1);//setta i canali: ch0 f=868.1, ecc. i primi 3 sono quelli obbligatori 
    lora.setChannel(1, 868.3);
    lora.setChannel(2, 868.5);
    /*Dopo aver trasmesso in uplink un frame un end device (o Mote) deve attendere la ricezione di una risposta;
     * a tale scopo sono definiti due intervalli di tempo (timeslot) detti finestre di ricezione entro cui l'end node deve attendere 
     * la ricezione di messaggi in downlink
     */
    lora.setReceiceWindowFirst(0, 868.1);
    lora.setReceiceWindowSecond(869.5, DR3);
    
    lora.setDutyCycle(false); //il duty cycle va fissato per la modalità OTAA e specifica l'intervallo di tempo rispetto a quello totale disponibile 
                              //in cui una risorsa è disponibile, cioè si trasmette 
    lora.setJoinDutyCycle(false);
    
    lora.setPower(14);//setta l'EIRP di trasmissione, il cui valore massimo è 14 dBm
}

void loop()
{   
        
  delay(10*1000); // Attendi 10 s fra due misure
    SerialUSB.println("---------------------------------------------------------------------------------------");
    SerialUSB.println("Valori di temperatura e umidità rilevati da DHT11 e inviati al network server loriot.io");
    SerialUSB.println("(con esito della trasmissione):  ");
  lora.transferPacket("Temperatura e umidita da DHT11:"); //trasmette al network server l'indicazione sul tipo di sensore usato

  temp = dht.readTemperature(); // legge la temperatura dal sensore
  
  trasmette(temp);
  //richiama la funzione che trasmette al network server, tramite il gateway Rasperry PI, il dato di temperatura rilevato, convertito in carattere e inviato in esadecimale
   
  umid = dht.readHumidity(); //legge l'umidità; ci vogliono circa 250 ms per leggere l'umidità o la temperatura
  
  trasmette(umid); 
  //richiama la funzione che trasmette al network server, tramite il gateway Rasperry PI, il dato di umidità rilevato, convertito in carattere e inviato in esadecimale
  SerialUSB.println("------------------------------------------------------------------------------------------");
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//funzione che trasmette i dati al network server loriot, tramite il gateway Rasperry PI, 
//convertendo il tipo da float a carattere; i caratteri sono inviati in esadecimale

void trasmette(float datofloat){  
    // ---- converte i dati acquisiti da float a carattere
    char buffer1[16];
    String str = String(datofloat, 2);
    str.toCharArray(buffer1,16);
    char* datoascii = buffer1;
    //-------------------
    SerialUSB.println(datoascii);
    bool esito = lora.transferPacket(datoascii); //trasmette il dato al network server tramite il gateway
    //la risposta all'esito della trasmissione è un bool --> se OK result = true = 1
    if(esito)
    {
        short lunghezza;
        short rssi;
        memset(buffer, 0, 256);
        lunghezza = lora.receivePacket(buffer, 256, &rssi);
        //riceve e invia sul monitor seriale le eventuali risposte, contenti anche l'RSSI (Radio Strenght Signal Indicator)
        
        if(lunghezza)
        {
            //SerialUSB.print("La risposta è lunga: ");
            //SerialUSB.println(lunghezza);
            SerialUSB.print("Il valore di RSSI è: ");
            SerialUSB.println(rssi);
            SerialUSB.print("I dati ricevuti sono: ");
            for(unsigned char i = 0; i < lunghezza; i ++)
            {
                SerialUSB.print("0x");
                SerialUSB.print(buffer[i], HEX);
                SerialUSB.print(" ");
            }
            SerialUSB.println();
        }
        }
    }
