/* Sketch a corredo delle risorse online del libro di testo
   Onelio Bertazioli
   Telecomunicazioni - Seconda edizione
   Zanichelli*/
//Invio al network server loriot.io dei dati di temperatura acquisiti con il sensore LM35 tramite seeeduino e gateway Raspberry Pi 
//seeeduino + sensore costituisce un end node IoT, che viene anche denominato Mote

//---------------------------------------------------------------------------------------------------------
#include <LoRaWan.h> 
// La libreria LoRaWan gestisce la comunicazione via LoRaWan
//-----------------------------------------------------------------------------------------------------------
#define pin_input A2  //denominiamo pin_input il pin A2 (opzionale)
int val_conv; //numero restituito dal convertitore A/D 
float temp; //temperatura
int temperatura;
char * tempascii;
char buffer[256];

//----------------------------------------
void setup(void)
{
    SerialUSB.begin(9600);
    while(!SerialUSB);
    
    lora.init(); //inizializza il modem LoRa
    //configura seeeduino per la trasmissione e manda sul monitor seriale le informazioni relative
    memset(buffer, 0, 256);
    lora.getVersion(buffer, 256, 1);
        SerialUSB.print(buffer); 
        memset(buffer, 0, 256);
    lora.getId(buffer, 256, 1);
    SerialUSB.print(buffer);
    
     lora.setKey("lavostraNWKKEY", "lavostraAPPSKEY", NULL);
    //Inserire le proprie Chiavi NWKKEY, APPSKEY, (NULL significa che APPKEY NON viene usata nella modalità APB, Activation By Personalization) da configurare 
    //sia su seeduino sia su loriot.io per configurare una sessione ABP (Activate By Personalization) su loriot.io)
    
    lora.setDeciveMode(LWABP);
    //setta la modalità ABP (Activation By Personalization) che fa unire direttamene un end-device a una rete lorawan saltando la fase di JOIN della modalità OTAA
    
    lora.setDataRate(DR0, EU868); //setta il data rate DR0 (250 bit/s), che consente di operare alla massima distanza dal gateway; 
    //però, se è attiva la modalità ADR (Adaptive Data Rate) il data rate può variare per adattarsi alla qualità del canale trasmissivo, aumentando e diminuiendo il data rate 
    
    lora.setChannel(0, 868.1);//setta i canali radio obbligatori: ch0 f=868.1, ecc.  
    lora.setChannel(1, 868.3);
    lora.setChannel(2, 868.5);
    
    /*Dopo aver trasmesso in uplink un frame un end device (o Mote) deve attendere la ricezione di una risposta;
     * a tale scopo sono definiti due intervalli di tempo (timeslot) detti finestre di ricezione entro cui l'end node deve attendere 
     * la ricezione di messaggi in downlink
     */
    lora.setReceiceWindowFirst(0, 868.1);
    lora.setReceiceWindowSecond(869.5, DR3);
    
    lora.setDutyCycle(false); //il duty cycle va fissato per la modalità OTAA e specifica l'intervallo di tempo rispetto a quello totale tra due trasmissioni Duty Cycle = Ton/(Ton+Toff)
                               
    lora.setJoinDutyCycle(false);
    
    lora.setPower(14);//setta l'EIRP di trasmissione al valore massimo consentito che a 868 MHz è pari a EIRPmax= +14 dBm
}

void loop(void)
{   
    val_conv=analogRead(pin_input);
    temp=(330.0/1023.0)*val_conv; //acquisisce la temperatura dal sensore LM35

    //---------- il dato di temperatura viene convertito da float a caratteri ascii per poter essere inviato in formato esadecimale al network server Loriot.io
    char buffer1[16];
    String str = String(temp, 2);
    str.toCharArray(buffer1,16);
    tempascii = buffer1;
    //-----------------------------------------------------
    
    SerialUSB.println("-----------------------------------------");
    SerialUSB.print("Valore di temperatura inviato al server:  ");
    SerialUSB.println(tempascii);
    SerialUSB.println("-----------------------------------------");
    
    bool esito = false;
    //dopo aver trasmesso l'end device riceve la risposta del gateway che è una variabile booleana, se è zero trasmissione fallita, se è uno trasmissione andata a buon fine  
    
    //--- Trasmette al network server loriot.io l'informazione sul sensore usato e il dato di temperatura acquisito e convertito in carattere
    esito = lora.transferPacket("Temperatura misurata da LM35:"); //testo di commento inviato
    delay(10);
    esito = lora.transferPacket(tempascii); //dato di temperatura inviato
    
    //la risposta all'esito della trasmissione è un bool --> se OK result = true = 1
    SerialUSB.print("Esito:   ");
    SerialUSB.println(esito);

    if(esito)
    {
        short lunghezza;
        short rssi;
        
        memset(buffer, 0, 256);
        lunghezza = lora.receivePacket(buffer, 256, &rssi);
        //seeduino può ricevere o meno una risposta dal gateway  
        //contente anche l'RSSI (Radio Strenght Signal Indicator), che viene visualizzata sul monitor seriale
        
        SerialUSB.print("Lunghezza della risposta:   ");
        SerialUSB.println(lunghezza);
        if(lunghezza)
        {
            SerialUSB.print("La risposta è lunga: ");
            SerialUSB.println(lunghezza);
            SerialUSB.print("Il valore di RSSI è: ");
            SerialUSB.println(rssi);
            SerialUSB.print("I dati ricevuti sono: ");
            for(unsigned char i = 0; i < lunghezza; i ++)
            {
                SerialUSB.print("0x");
                SerialUSB.print(buffer[i], HEX);
                SerialUSB.print(" ");
            }
            SerialUSB.println();
        }
    }
}
