/*
  Misura di umidità e temperatura con il sensore DHT22 e visualizzazione sull'applicazione 
  Android “Bluetooth Electronics”, pannello "Temperature and Humidity Demo".
  I dati di umidità e temperatura sono contenuti nei bit 1-16 e 17-32 della stringa inviata dal sensore.
*/

#include <SoftwareSerial.h>        // libreria comunicazione verso il modulo HC-XX
SoftwareSerial SerialBlue (2, 4); // collegato con i pin di Arduino: 2=RX ; 4=TX
                                  
int dataPin = 9;
boolean stringaRic[41];    // stringa ricevuta dal DHT22 (compreso bit di start)
unsigned int temp;            // temperature in °C
unsigned int humidity;     // umidità relativa (%RH) 

void setup() {

  SerialBlue.begin(9600); // inizializza comunicazione via Bluetooth
  Serial.begin(9600);        // inizializza Serial Monitor
}

void loop() {
  
  delay (2000);   // pausa di 2 s prima di una nuova richiesta di misura
 
          // porta basso il pin DATA per 18 ms per richiedere nuovi dati 
  pinMode (dataPin, OUTPUT);    // pin 9 OUTPUT (da Arduino a DHT22)
  digitalWrite (dataPin,LOW);
  delay(18);
  digitalWrite (dataPin,HIGH);
    
  pinMode (dataPin, INPUT);    // riporta il pin 9 in modalità INPUT per ricevere i dati dal DHT22 
  
  // leggi 41 bit dal DHT22 (il 1° impulso di start va poi ignorato)
  for (int i=0; i<=40; i++){
    stringaRic[i]=(pulseIn(dataPin, HIGH)>40);  // pulseIn restituisce 
                                         		 // la durata in microsec. dell’impulso a livello alto, 
    }                                     		// se è >40 microsec. il bit è un 1, altrimenti è 0
    
          // ricava il valore di umidità dai bit 1-16 dell’array stringaRic
  humidity=0;
  for (int i=1; i<=16; i++){     // leggi i 16 bit dell’umidità
    humidity=humidity<<1;        // shift SX di una posizione del valore binario di humidity 
    if (stringaRic[i]) humidity |= 1; // bitwise compound OR: se l’iesimo bit trasmesso è 1, questo viene sostituito 
                                      // allo 0 nel LSB di humidity, altrimenti rimane 0
  }                                                
    humidity=humidity/10;      // per ottenere il valore reale si deve dividere per 10 quello ottenuto dalla stringa
       
   // ricava il valore di temperatura dai bit 17-32 dell’array stringaRic
  temp=0;                                          // stesso procedimento di humidity
  for (int i=17;i<=32;i++){
    temp=temp<<1;     
    if (stringaRic[i]) temp|=1;     
  }
  temp=temp/10;
 
          // trasmetti i dati su Bluetooth in formato ‘*Txx*’ e ‘*Hxx*’
  SerialBlue.print ("*T"+String(temp)+"*");          // trasmetti temperatura
  SerialBlue.print ("*H"+String(humidity)+"*");   // trasmetti umidità

  // scrivi su Serial Monitor i 40 bit della stringa e i valori trasmessi su Bluetooth
  for (int i=0;i<=40;i++){                   
    Serial.print (stringaRic[i]);                    
    if (i==0 || i==16 || i==32) Serial.print (" ");
  } 
  Serial.println (" ");   
  Serial.print ("  *H"+String(humidity)+"*"); 
  Serial.print ("            ");
  Serial.println ("*T"+String(temp)+"*");
}
