//***************************************************************************
//          Sketch a corredo del libro di testo                             *
// Onelio Bertazioli - Telecomunicazioni Seconda edizione - ed- Zanichelli  *
//***************************************************************************
/*  Misura di temperatura e umidità con il sensore DHT11 e invio dei valori misurati a
 *  un broker MQTT per la pubblicazione; il nome del topic scelto è LAB-IOT/ARDUINO-DHT11
 *  come broker/server MQTT di test si usa quello pubblico "test.mosquitto.org"
 *  i valori possono essere letti con un client MQTT sia a linea di comando, installando mosquitto-clients, sia 
 *  con interfaccia grafica installando WMQTT (applicazione JAVA) sia utilizzando un client MQTT scritto in Python*/
//-------       librerie per la connessione Ethernet e al broker MQTT
#include <Ethernet.h>
#include <PubSubClient.h>
/*----------------------- LIBRERIE PER IL SENSORE DHT 11 ----------------------------
-> DHT Sensor Library: https://github.com/adafruit/DHT-sensor-library
-> Adafruit Unified Sensor Lib: https://github.com/adafruit/Adafruit_Sensor  */
#include <DHT.h>
#include <DHT_U.h>
//-------------------------------------------------------------------------------------
#define DHTTYPE  DHT11    //specifica il tipo di sensore DHT
#define DHTPIN 2   // specifica il pin di Arduino a cui è collegato il pin Signal del DHT11 
DHT dht(DHTPIN, DHTTYPE);

float temp; //valore temperatura
float umid; //valore umidità
//----------------------------------------------------------------------------------------------------
byte mac[]={0x90, 0xAA,0xAB, 0xAC, 0xAD, 0xAE}; //mettere qui l'indirizzo mac dello shield Ethernet anteponendo a ogni coppia di cifre 0x
//----------------------------------------------------------------------------------------------------
#define brokerMQTT "test.mosquitto.org"      //server/broker MQTT pubblico per le prove
#define NOMETOPIC "LAB-IOT/ARDUINO-DHT11"    //nome del topic
#define CLIENTID "Arduino-Uno"            //identificativo scelto
//----------------------------------------------------------------
//Temperatura e umidità in formato carattere per la pubblicazione
char * carTemperatura;
char * carUmid;
//Inizializzazione dei client Ethernet e MQTT
EthernetClient Client_ethernet; //inizializza il client Ethernet
PubSubClient arduinoClient(brokerMQTT,1883,Client_ethernet);     //Crea un'istanza e inizializza il client MQTT
//
// ---FUNZIONE CHE CONVERTE I VALORI DA FLOAT A CARATTERE PER L'INVIO AL BROKER MQTT ------------
char buffer1[16];
String str;
char * converti(float datofloat){
    buffer1[16];
    str = String(datofloat, 2);
    str.toCharArray(buffer1,16);
    char * datoascii = buffer1;
    return datoascii;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void setup() {

  dht.begin();  // Inizializza il sensore

  Serial.begin(9600);
  beginConnection() ; //Effettua la connessione al server/broker MQTT
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void loop(void) {
// lettura della tensione sul pin a cui è collegato il sensore DHT11
  temp = dht.readTemperature(); // legge la temperatura dal sensore
  carTemperatura = converti(temp); //valore di temperatura in formato carattere

// Trasmetti al broker MQTT un messaggio e la temperatura rilevata del sensore
  arduinoClient.publish(NOMETOPIC, "Temperatura (°C) misurata dal sensore DHT11 su Arduino Uno:", true) ;
  arduinoClient.publish(NOMETOPIC, carTemperatura, true) ; //invia al broker MQTT il valore della temperatura da pubblicare, con impostato il retain (il broker mantiene l'ultimo messaggio)

// Visualizza anche sul monitor seriale
  Serial.print("Temperatura (°C) misurata dal sensore DHT11 su Arduino Uno:  ");
  Serial.println(carTemperatura);
  delay(300);
//
  umid = dht.readHumidity(); //legge l'umidità; ci vogliono circa 250 ms per leggere l'umidità o la temperatura
  carUmid = converti(umid); //valore di umidità in formato carattere

// Trasmetti al broker MQTT un messaggio e l'umidità rilevata del sensore
  arduinoClient.publish(NOMETOPIC, "Umidita' (%) misurata dal sensore DHT11 su Arduino Uno:", true) ; //invia al broker MQTT il messaggio
  arduinoClient.publish(NOMETOPIC, carUmid, true) ; //invia al broker MQTT il valore dell'umidità da pubblicare, con impostato il retain (true, il broker mantiene l'ultimo messaggio)

//
  Serial.print("Umidità (%) misurata dal sensore DHT11 su Arduino Uno:  ");
  Serial.println(carUmid);
  Serial.println("---------------------------------------------------------------------");
  delay(10000); //aspetta 10 secondi prima di inviare delle nuove misure
}
//
//----------------  Funzione che gestisce la connessione con il broker/server MQTT --------------------
void beginConnection() {
//Ethernet.begin(mac,ip) ; // se si vuole usare un indirizzo IP statico
  Ethernet.begin(mac); //usare questo se si vuole acquisire l'IP con DHCP; bisona però mettere un delay dopo per aspettare che venga acquisito l'IP
//
  int RichiestaConn = arduinoClient.connect(CLIENTID) ;

  delay(10000) ; //aspetta (10 s) che il DHCP assegni l'indirizzo IP e che il client si connetta
  if (!RichiestaConn) {    //se la connessione non è avvenuta RichiestaConn = 0 segnala ciò
    Serial.println(RichiestaConn) ;
    Serial.println("Non riesco a connettermi al server/broker MQTT");
    Serial.println("Controllare la configurazione di Arduino");
    delay(100);
    exit(-1);
  }
  else {
    Serial.println("*****************************************************************************");
    Serial.print("Connesso al server/broker MQTT:  ");
    Serial.println(brokerMQTT);
    Serial.print("Invio dei dati al topic MQTT:  ");
    Serial.println(NOMETOPIC);
    Serial.println("*****************************************************************************");
  }
}
