//***************************************************************************
//          Sketch a corredo del libro di testo                             *
// Onelio Bertazioli - Telecomunicazioni Seconda edizione - ed- Zanichelli  *
//***************************************************************************
/*  Misura di temperatura e umidità con il sensore LM35 e invio dei valori misurati a
 *  un broker MQTT per la pubblicazione; il nome del topic scelto è LAB-IOT/ARDUINO-LM35
 *  come broker/server MQTT di test si usa quello pubblico "test.mosquitto.org"
 *  i valori possono essere letti con un client MQTT sia con interfaccia grafica (come WMQTT) 
 *  sia utilizzando il client MQTT scritto in Python scaricabile dalle risorse online del testo*/
//-------       librerie per la connessione Ethernet e al broker MQTT
#include <SPI.h>
#include <Ethernet.h>
#include <PubSubClient.h>
//------------------- sensore LM35 ---------------------
#define pin_input A2  //denominiamo pin_input il pin A2 a cui è collegato il pin di uscita del sensore LM35
int val_conv_AD;     //numero restituito dal convertitore A/D 
float temp;          //temperatura misurata
//----------------------------------------------------------------------------------------------------
byte mac[]={0x90, 0xAA,0xAB, 0xAC, 0xAD, 0xAE}; //mettere qui l'indirizzo mac dello shield Ethernet anteponendo a ogni coppia di cifre 0x
//byte ip[]={10,0,0,188};                     // indirizzo IP statico da usare se non si vuole il DHCP
//----------------------------------------------------------------------------------------------------
#define brokerMQTT "test.mosquitto.org"      //server/broker MQTT pubblico per le prove
#define NOMETOPIC "LAB-IOT/ARDUINO-LM35"    //nome del topic
#define CLIENTID "Arduino-Uno"            //identificativo scelto per ilclient MQTT
//----------------------------------------------------------------
//Temperatura in formato carattere per la pubblicazione tramite il broker MQTT
char * carTemperatura;
// ---FUNZIONE CHE CONVERTE I VALORI DA FLOAT A CARATTERE PER L'INVIO AL BROKER MQTT ------------
char buffer1[16];
String str;
char * converti(float datofloat){
    buffer1[16];
    str = String(datofloat, 2);
    str.toCharArray(buffer1,16);
    char * datoascii = buffer1;
    return datoascii;
}
// ------   Inizializzazione dei client Ethernet e MQTT
EthernetClient Client_ethernet; //inizializza il client Ethernet
PubSubClient arduinoClient(brokerMQTT,1883,Client_ethernet);     //Crea un'istanza e inizializza il client MQTT
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void setup() {
  Serial.begin(9600);
  beginConnection() ; //Funzione che effettua la connessione al server/broker MQTT
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void loop(void) {
// lettura della tensione sul pin a cui è collegato il sensore LM35
  val_conv_AD=analogRead(pin_input);  //
  temp=(500.0/1023.0)*val_conv_AD;    //valore di temperatura misurato
  carTemperatura = converti(temp); //valore di temperatura in formato carattere
// Trasmetti al broker MQTT un messaggio e la temperatura rilevata del sensore
  arduinoClient.publish(NOMETOPIC, "Temperatura misurata dal sensore LM35 su Arduino Uno:", true) ;
  arduinoClient.publish(NOMETOPIC, carTemperatura, true) ; //invia al broker MQTT il valore della temperatura da pubblicare, con impostato il retain (il broker mantiene l'ultimo messaggio)
// Visualizza anche sul monitor seriale
  Serial.print("Temperatura misurata dal sensore LM35 su Arduino Uno:  ");
  Serial.println(carTemperatura);
  delay(300);
  Serial.println("---------------------------------------------------------------------");
  delay(10000); //aspetta 10 secondi prima di inviare delle nuove misure
}
//----------------  Funzione che gestisce la connessione con il broker/server MQTT --------------------
void beginConnection() {
//Ethernet.begin(mac,ip) ; // se si vuole usare un indirizzo IP statico
  Ethernet.begin(mac); //usare questo se si vuole acquisire l'IP da un server DHCP
//
  int RichiestaConn = arduinoClient.connect(CLIENTID) ;
  delay(10000) ; //aspetta (10 s) che il client si connetta
  if (!RichiestaConn) {    //se la connessione non è avvenuta RichiestaConn = 0 e segnala ciò
    Serial.println("Non riesco a connettermi al server/broker MQTT");
    Serial.println("Controllare la configurazione di Arduino");
    delay(100);
    exit(-1);
  }
  else {
    Serial.println("*****************************************************************************");
    Serial.print("Connesso al server/broker MQTT:  ");
    Serial.println(brokerMQTT);
    Serial.print("Invio dei dati al topic MQTT:  ");
    Serial.println(NOMETOPIC);
    Serial.println("*****************************************************************************");
  }
}
